import logging
from datetime import date, datetime, time

from django.utils import timezone
from dateutil.parser import parse
from djpsa.sync.sync import Synchronizer
from django.db.models.fields import DateTimeField
from django.db.models.fields.related import ForeignKey

# README #
#
# "response_key"
#    The Halo API is very inconsistent, the response_key field is used to
#     specify the key in the response that contains the data we want to unpack.
#
#     Where when the response is just a list with no key, the response_key is
#     omitted from the class. ResponseKeyMixin should be applied to any class
#     that requires the response_key field.
#
# "lookup_key"
#    Some records need to be tracked by a different field than id for the
#     primary key. For example, the priority model uses priorityid as the
#     primary key, so the lookup_key is set to 'priorityid'. This is because
#     in the Halo API the 'id' seems to be a large alphanumeric string, and
#     isn't used on the ticket model.

logger = logging.getLogger(__name__)


def empty_date_parser(date_time):
    # Halo API returns a date of 1/1/1900 or earlier as an empty date.
    # This will set the model fields as None if it is an impossible date.
    # Set to 1980 in case they also do 1950 or something and I haven't seen it.
    if date_time:
        try:
            date_time = timezone.make_aware(parse(date_time), timezone.utc)
        except ValueError:
            date_time = parse(date_time)
        return date_time if date_time.year > 1980 else None


def parse_date_from_api(date_time_str):
    # Halo returns date fields as datetime strings (e.g."2026-02-10T00:00:00")
    if not date_time_str:
        return None

    parsed_datetime = empty_date_parser(date_time_str)
    if not parsed_datetime:
        return None

    # Ensure it's UTC-aware
    if timezone.is_naive(parsed_datetime):
        parsed_datetime = timezone.make_aware(parsed_datetime, timezone.utc)

    # Extract the date portion
    return parsed_datetime.date()


def format_date_for_api(date_value):
    # Halo API expects date fields as datetime strings
    # without timezone indicators. We treat the date as
    # 12 noon in the server's timezone, convert to UTC, and format
    # as an ISO string without timezone information.
    if not date_value:
        return None

    # Convert string to date if needed
    if isinstance(date_value, str):
        date_value = date.fromisoformat(date_value)

    # Treat as 12 noon in the server's timezone
    server_tz = timezone.get_current_timezone()

    server_noon = timezone.make_aware(
        datetime.combine(date_value, time(hour=12)),
        server_tz
    )

    utc_noon = server_noon.astimezone(timezone.utc)
    # Format as ISO string without timezone indicator
    return utc_noon.strftime('%Y-%m-%dT%H:%M:%S')


class ResponseKeyMixin:
    response_key = None

    def _unpack_records(self, response):
        records = response[self.response_key]
        return records


class ApiConvertMixin:
    client = None
    model_class = None

    def _convert_fields_to_api_format(self, data):
        """
        Converts the model field names to the API field names.
        """
        api_data = {}
        for key, value in data.items():
            api_data[self.model_class.API_FIELDS[key]] = value
        return api_data

    def _convert_fields(self, data):
        """
        Convert field values as necessary:
        * Datetime fields to ISO string format.
        * Team just wants a name, not the ID.
        * Foreign keys to the ID of the related object.
        """
        for key, value in data.items():
            field_class = self.model_class._meta.get_field(key).__class__
            if field_class == DateTimeField:
                data[key] = value.isoformat() if value else None
            elif key == 'team':
                # Team requires the name of the team, not the ID. :rageguy:
                data[key] = value.name if value else None
            elif field_class == ForeignKey:
                try:
                    data[key] = value.id if value else None
                except AttributeError:
                    # The field is a string, not a model instance.
                    data[key] = value
            else:
                data[key] = value
        return data


class CreateMixin(ApiConvertMixin):

    def create(self, data, *args, **kwargs):
        data = self._convert_fields(data)
        body = self._convert_fields_to_api_format(data)
        response = self.client.create(body)

        instance, _ = self.update_or_create_instance(response)
        return instance


class UpdateMixin(ApiConvertMixin):

    def update(self, record, data, *args, **kwargs):
        data = self._convert_fields(data)
        body = self._convert_fields_to_api_format(data)

        response = self.client.update(record.id, body)

        instance, _ = self.update_or_create_instance(response)
        return instance


class DeleteMixin:
    client = None

    def delete(self, record_id, *args, **kwargs):
        return self.client.delete(record_id)


class HaloSynchronizer(Synchronizer):

    def _format_job_condition(self, last_sync_time):
        return {
            self.last_updated_field: last_sync_time
        }


class HaloChildFetchRecordsMixin:
    parent_model_class = None
    parent_field = None

    def __init__(self, parent_object_id=None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.parent_object_id = parent_object_id

    def fetch_records(self, results, params=None):
        params = params or {}

        batch = 1
        for object_id in self.parent_object_ids:
            logger.info(
                'Fetching {} records, batch {}'.format(
                    self.get_model_name(), batch)
            )
            params.update(self.format_parent_params(object_id))
            response = self.client.fetch_resource(params=params)
            records = self._unpack_records(response)
            self.persist_page(records, results)
            batch += 1

        return results

    @property
    def parent_object_ids(self):
        object_ids = self.parent_model_class.objects.all() \
            .values_list('id', flat=True)

        if self.parent_object_id:
            object_ids = [self.parent_object_id]

        return object_ids

    def format_parent_params(self, object_id):
        return {
            self.parent_field: object_id
        }



from django.db import models
from model_utils import FieldTracker


class BudgetType(models.Model):
    name = models.CharField(max_length=255, blank=True, null=True)
    default_rate = models.DecimalField(
        max_digits=10, decimal_places=2, blank=True, null=True)

    class Meta:
        verbose_name_plural = "Budget Types"

    def __str__(self):
        return str(self.name)


class BudgetTypeTracker(BudgetType):
    tracker = FieldTracker()

    class Meta:
        proxy = True
        db_table = 'halo_budgettype'



# Generated by Django 4.2.11 on 2025-01-17 14:46

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('halo', '0020_budgettype_ticket_itil_request_type_budgetdata_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='budgetdata',
            options={'verbose_name_plural': 'Budget Data'},
        ),
    ]



from djpsa.halo import models
from djpsa.halo.records import api
from djpsa.halo import sync


class BudgetTypeSynchronizer(sync.HaloSynchronizer):
    model_class = models.BudgetTypeTracker
    client_class = api.BudgetTypeAPI

    def _assign_field_data(self, instance, json_data):
        instance.id = json_data.get('id')
        instance.name = json_data.get('name')
        instance.default_rate = json_data.get('defaultrate')



from djpsa.halo.records import api


class BudgetDataAPI(api.TicketAPI):
    # Not a real endpoint, just an easy way to get
    # budgets off of a ticket

    def get(self, record_id):
        return self.request(
            'GET', endpoint_url=self._format_endpoint(record_id))



# Generated by Django 4.2.17 on 2025-03-21 15:29

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('halo', '0027_alter_action_charge_rate_alter_ticket_deadline_date_and_more'),
        ('halo', '0027_alter_ticket_appointment_type_and_more'),
    ]

    operations = [
    ]



# Generated by Django 4.2.11 on 2024-09-24 09:57

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('halo', '0010_agenttracker_appointmenttracker_clienttracker_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='TicketType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('agents_can_select', models.BooleanField(default=False)),
                ('end_users_can_select', models.BooleanField(default=False)),
                ('allow_attachments', models.BooleanField(default=False)),
                ('can_create', models.BooleanField(default=False)),
                ('copy_attachments_to_child', models.BooleanField(default=False)),
                ('copy_attachments_to_related', models.BooleanField(default=False)),
                ('use', models.CharField(blank=True, max_length=255, null=True)),
            ],
        ),
        migrations.RemoveField(
            model_name='project',
            name='agent',
        ),
        migrations.RemoveField(
            model_name='project',
            name='client',
        ),
        migrations.RemoveField(
            model_name='project',
            name='priority',
        ),
        migrations.RemoveField(
            model_name='project',
            name='site',
        ),
        migrations.RemoveField(
            model_name='project',
            name='status',
        ),
        migrations.RemoveField(
            model_name='project',
            name='user',
        ),
        migrations.DeleteModel(
            name='ProjectTracker',
        ),
        migrations.AddField(
            model_name='client',
            name='colour',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='client',
            name='main_site',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='halo.site'),
        ),
        migrations.AddField(
            model_name='client',
            name='phone_number',
            field=models.CharField(blank=True, max_length=250, null=True),
        ),
        migrations.AddField(
            model_name='ticket',
            name='appointment_type',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='ticket',
            name='impact_level',
            field=models.IntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='ticket',
            name='project',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='halo.ticket'),
        ),
        migrations.DeleteModel(
            name='Project',
        ),
        migrations.CreateModel(
            name='TicketTypeTracker',
            fields=[
            ],
            options={
                'db_table': 'halo_tickettype',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('halo.tickettype',),
        ),
        migrations.AddField(
            model_name='ticket',
            name='type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='halo.tickettype'),
        ),
    ]



from django.db import models
from model_utils import FieldTracker


class Action(models.Model):

    # start time
    action_arrival_date = models.DateTimeField(blank=True, null=True)

    # end time
    action_completion_date = models.DateTimeField(blank=True, null=True)

    action_date_created = models.DateTimeField(blank=True, null=True)
    time_taken = models.FloatField(blank=True, null=True)
    time_taken_adjusted = models.FloatField(blank=True, null=True)
    time_taken_days = models.FloatField(blank=True, null=True)
    non_billable_time = models.FloatField(blank=True, null=True)
    travel_time = models.FloatField(blank=True, null=True)
    note = models.TextField(blank=True, null=True)
    action_charge_amount = models.FloatField(blank=True, null=True)
    action_charge_hours = models.FloatField(blank=True, null=True)
    action_non_charge_amount = models.FloatField(blank=True, null=True)
    action_non_charge_hours = models.FloatField(blank=True, null=True)
    act_is_billable = models.BooleanField(default=False)
    attachment_count = models.IntegerField(blank=True, null=True)
    hidden_from_user = models.BooleanField(default=False)
    important = models.BooleanField(default=False)

    ticket = models.ForeignKey(
        'Ticket',
        blank=True,
        null=True,
        on_delete=models.CASCADE,
        related_name='ticket_actions'
    )
    project = models.ForeignKey(
        'Ticket',
        blank=True,
        null=True,
        on_delete=models.CASCADE,
        related_name='project_actions'
    )
    agent = models.ForeignKey(
        'Agent', blank=True, null=True, on_delete=models.CASCADE)
    outcome = models.ForeignKey(
        'Outcome', blank=True, null=True, on_delete=models.CASCADE)
    charge_rate = models.ForeignKey(
        'ChargeRate', blank=True, null=True, on_delete=models.CASCADE)

    class Meta:
        verbose_name_plural = "Actions"

    def __str__(self):
        return f"Action {self.id}"

    API_FIELDS = {
        'id': 'id',
        'action_arrival_date': 'actionarrivaldate',
        'action_completion_date': 'actioncompletiondate',
        'time_taken': 'timetaken',
        'time_taken_adjusted': 'timetakenadjusted',
        'time_taken_days': 'timetakendays',
        'non_billable_time': 'nonbilltime',
        'travel_time': 'traveltime',
        'note': 'note',
        'action_charge_amount': 'actionchargeamount',
        'action_charge_hours': 'actionchargehours',
        'action_non_charge_amount': 'actionnonchargeamount',
        'action_non_charge_hours': 'actionnonchargehours',
        'act_is_billable': 'actisbillable',
        'attachment_count': 'attachment_count',
        'charge_rate': 'chargerate',
        'hidden_from_user': 'hiddenfromuser',
        'important': 'important',
        'ticket': 'ticket_id',
        'project': 'project_id',
        'agent': 'who',
        'outcome': 'outcome_id',
    }


class ActionTracker(Action):
    tracker = FieldTracker()

    class Meta:
        proxy = True
        db_table = 'halo_action'



from djpsa.halo.records import api
from djpsa.halo import sync, models
from djpsa.halo.records.ticket.sync import TicketSynchronizer


class TimeSheetEventSynchronizer(
            sync.ResponseKeyMixin,
            sync.CreateMixin,
            sync.HaloSynchronizer,
        ):
    model_class = None
    client_class = api.TimesheetEventAPI
    # For now, these are also the only fields we will support. We will
    # need to revisit this when we also improve action creation.
    required_fields = [
        'start_date',
        'end_date',
        'ticket_id',
        'agent_id',
        'charge_rate',
        'note'
    ]

    def create(self, data, *args, **kwargs):
        """
        Example data format:
        {
          "end_date": "2025-02-25T18:26:00.000Z",
          "start_date": "2025-02-25T18:10:00.000Z",
          "ticket_id": 2267,
          "agent_id": "3",
          "charge_rate": "1",
          "note": "log time action test, delete me",
        }
        """
        for field in self.required_fields:
            if field not in data:
                raise ValueError(f"Missing required field: {field}")

        # Convert datetimes to string format.
        data['start_date'] = data['start_date'].isoformat()
        data['end_date'] = data['end_date'].isoformat()

        # Don't create a new ticket when logging time
        data['lognewticket'] = False

        # This is the event type for a quick time entry... I think...
        # It's probably one of the LookupType system events, but I can't find
        # it. However, this is what Halo uses when you do a quick time entry
        # from the time sheet, so that's what we'll use.
        data['event_type'] = 0

        self.client.create(data)

        ticket = models.Ticket.objects.get(id=data['ticket_id'])

        # Sync related to pull down the newly created action
        sync = TicketSynchronizer()
        sync.sync_related(ticket)



from django.db import models
from model_utils import FieldTracker


class Status(models.Model):
    name = models.CharField(max_length=255)
    colour = models.CharField(max_length=255, blank=True, null=True)

    class Meta:
        verbose_name_plural = "Statuses"

    def __str__(self):
        return str(self.name)


class StatusTracker(Status):
    tracker = FieldTracker()

    class Meta:
        proxy = True
        db_table = 'halo_status'



from django.db import models
from model_utils import FieldTracker


class SLA(models.Model):

    name = models.CharField(max_length=255)
    hours_are_techs_local_time = models.BooleanField(default=False)
    response_reset = models.BooleanField(default=False)
    response_reset_approval = models.BooleanField(default=False)
    track_sla_fix_by_time = models.BooleanField(default=False)
    track_sla_response_time = models.BooleanField(default=False)
    workday_id = models.IntegerField(blank=True, null=True)
    auto_release_limit = models.IntegerField(blank=True, null=True)
    auto_release_option = models.BooleanField(default=False)
    status_after_first_warning = models.IntegerField(blank=True, null=True)
    status_after_second_warning = models.IntegerField(blank=True, null=True)
    status_after_auto_release = models.IntegerField(blank=True, null=True)

    class Meta:
        verbose_name_plural = "SLAs"

    def __str__(self):
        return str(self.name)


class SLATracker(SLA):
    tracker = FieldTracker()

    class Meta:
        proxy = True
        db_table = 'halo_sla'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

LONG_DESCRIPTION = open('README.md').read()

VERSION = (0, 21, '1a0')

# pragma: no cover
if VERSION[-1] != "final":
    project_version = '.'.join(map(str, VERSION))
else:
    # pragma: no cover
    project_version = '.'.join(map(str, VERSION[:-1]))

setup(
    name="django-psa",
    version=project_version,
    description='Django app for working with '
                'various PSA REST API. Defines '
                'models (tickets, companies, '
                'etc.) and callbacks. ',
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    keywords='django connectwise halo autotask rest api python',
    packages=find_packages(),
    author='TopLeft Technologies Ltd.',
    author_email='sam@topleft.team',
    url="https://github.com/topleft-team/django-psa",
    include_package_data=True,
    license='MIT',
    install_requires=[
        'requests',
        'django',
        'setuptools',
        'python-dateutil',
        'retrying',
        'redis',
        'django-extensions',
    ],
    test_suite='runtests.suite',
    tests_require=[
        'names',
        'coverage',
        'flake8',
        'django-test-plus',
        'mock',
        'freezegun',
        'responses',
        'model-mommy',
        'django-coverage',
        'names',
        'django-environ'
    ],
    # Django likes to inspect apps for /migrations directories, and can't if
    # package is installed as an egg. zip_safe=False disables installation as
    # an egg.
    zip_safe=False,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Development Status :: 3 - Alpha',
    ],
)



from django.core.management.base import BaseCommand, CommandError
from django.conf import settings

from djpsa.api.exceptions import APIClientError


class Command(BaseCommand):
    help = 'Ensure callbacks are registered.'

    def handle(self, *args, **options):
        handler = settings.PROVIDER.callback_handler()
        try:
            handler.ensure_registered()
        except APIClientError as e:
            raise CommandError(e)



from enum import Enum

from django.db import models
from django_extensions.db.models import TimeStampedModel
from model_utils import FieldTracker


class ItilRequestType(Enum):
    # System defined ITIL request types. We can use these to identify
    # the type of ticket we are dealing with. This is useful for
    # when an instance has custom ticket types beyond the default.
    INCIDENT = 1
    CHANGE_REQUEST = 2
    SERVICE_REQUEST = 3
    PROBLEM = 4
    REQUEST_FOR_QUOTE = 20
    ADVICE_OTHER = 21
    PROJECTS = 22
    TASKS = 23


class TicketOnlyManager(models.Manager):
    """
    Tickets are tickets that are not in the appropriate ITIL request type
    and don't have a parent project.
    """
    def get_queryset(self):
        return super().get_queryset().exclude(
            itil_request_type=ItilRequestType.PROJECTS.value,
            project=None,
        )


class ProjectOnlyManager(models.Manager):
    """
    Projects are tickets with the appropriate ITIL request type
    and have no relation to a parent project.

    This should match the logic of the is_project method.
    """
    def get_queryset(self):
        return super().get_queryset().filter(
            itil_request_type=ItilRequestType.PROJECTS.value,
            project=None,
        )


class Ticket(TimeStampedModel):
    summary = models.CharField(blank=True, null=True, max_length=255)
    details = models.TextField(
        blank=True, null=True
    )
    status = models.ForeignKey('Status', on_delete=models.CASCADE)
    priority = models.ForeignKey(
        'Priority', blank=True, null=True, on_delete=models.CASCADE)
    client = models.ForeignKey(
        'Client', blank=True, null=True, on_delete=models.CASCADE)
    agent = models.ForeignKey(
        'Agent', blank=True, null=True, on_delete=models.CASCADE)
    sla = models.ForeignKey(
        'SLA', blank=True, null=True, on_delete=models.CASCADE)
    user = models.ForeignKey(
        'HaloUser', blank=True, null=True, on_delete=models.CASCADE,
        verbose_name='Contact',
    )
    site = models.ForeignKey(
        'Site', blank=True, null=True, on_delete=models.CASCADE)
    type = models.ForeignKey(
        'TicketType', blank=True, null=True, on_delete=models.CASCADE)
    project = models.ForeignKey(
        'Ticket', blank=True, null=True, on_delete=models.CASCADE)
    team = models.ForeignKey(
        'Team', blank=True, null=True, on_delete=models.CASCADE)
    user_email = models.EmailField(blank=True, null=True)
    reported_by = models.CharField(max_length=255, blank=True, null=True)
    end_user_status = models.IntegerField(blank=True, null=True)
    category_1 = models.CharField(max_length=255, blank=True, null=True)
    category_2 = models.CharField(max_length=255, blank=True, null=True)
    category_3 = models.CharField(max_length=255, blank=True, null=True)
    category_4 = models.CharField(max_length=255, blank=True, null=True)
    inactive = models.BooleanField(default=False)
    sla_response_state = \
        models.CharField(max_length=255, blank=True, null=True)
    sla_hold_time = models.FloatField(blank=True, null=True)
    date_occurred = models.DateTimeField(blank=True, null=True)
    respond_by_date = models.DateTimeField(blank=True, null=True)
    fix_by_date = models.DateTimeField(
        blank=True, null=True,
        verbose_name='Fix by',
    )
    date_assigned = models.DateTimeField(blank=True, null=True)
    response_date = models.DateTimeField(blank=True, null=True)
    deadline_date = models.DateTimeField(
        blank=True, null=True,
        verbose_name='Deadline',
    )
    last_action_date = models.DateTimeField(blank=True, null=True)
    last_update = models.DateTimeField(blank=True, null=True)
    date_closed = models.DateTimeField(blank=True, null=True)

    # Target and Start date are actually 2 fields from Halo,
    # but we are syncing both into the same field in the model.
    target_date = models.DateField(blank=True, null=True)
    start_date = models.DateField(blank=True, null=True)

    last_incoming_email = models.DateTimeField(blank=True, null=True)
    impact = models.IntegerField(blank=True, null=True)
    impact_level = models.IntegerField(blank=True, null=True)
    flagged = models.BooleanField(default=False)
    on_hold = models.BooleanField(default=False)
    project_time_actual = models.FloatField(blank=True, null=True)
    project_money_actual = models.FloatField(blank=True, null=True)
    cost = models.FloatField(blank=True, null=True)
    estimate = models.FloatField(
        blank=True, null=True,
        help_text='Effort in hours',
    )
    estimated_days = models.FloatField(blank=True, null=True)
    exclude_from_sla = models.BooleanField(default=False)
    reviewed = models.BooleanField(default=False)
    read = models.BooleanField(default=False)
    email_to_list = models.TextField(blank=True, null=True)
    urgency = models.IntegerField(blank=True, null=True)
    service_status_note = \
        models.TextField(blank=True, null=True)
    ticket_tags = models.TextField(blank=True, null=True)
    appointment_type = models.IntegerField(blank=True, null=True)
    itil_request_type = models.IntegerField(
        choices=[(
            e.value,
            e.name.replace('_', ' ').title()
        ) for e in ItilRequestType],
        default=ItilRequestType.INCIDENT.value,
        verbose_name='ITIL request type'
    )

    use = models.CharField(max_length=255, blank=True, null=True)

    API_FIELDS = {
        "id": "id",
        "summary": "summary",
        "details": "details",
        "last_action_date": "lastactiondate",
        "last_update": "last_update",
        "date_closed": "dateclosed",
        "user_email": "useremail",
        "reported_by": "reportedby",
        "end_user_status": "enduserstatus",
        "category_1": "category1",
        "category_2": "category2",
        "category_3": "category3",
        "category_4": "category4",
        "inactive": "inactive",
        "impact": "impact",
        "deadline_date": "deadlinedate",
        "flagged": "flagged",
        "on_hold": "onhold",
        "cost": "cost",
        "estimate": "estimate",
        "estimated_days": "estimateddays",
        "exclude_from_sla": "excludefromslas",
        "team": "team",
        "read": "read",
        "use": "use",
        "email_to_list": "emailtolist",
        "urgency": "urgency",
        "service_status_note": "servicestatusnote",
        "ticket_tags": "tickettags",
        "appointment_type": "appointment_type",
        "impact_level": "impactlevel",
        "fix_by_date": "fixbydate",
        "start_date": "startdate",
        "target_date": "targetdate",
        "last_incoming_email": "lastincomingemail",
        "client": "client_id",
        "status": "status_id",
        "priority": "priority_id",
        "agent": "agent_id",
        "sla": "sla_id",
        "user": "user_id",
        "site": "site_id",
        "type": "tickettype_id",
        "itil_request_type": "itil_requesttype_id",
        "project": "parent_id",
    }

    objects = models.Manager()
    tickets_only = TicketOnlyManager()
    projects_only = ProjectOnlyManager()

    class Meta:
        verbose_name_plural = "Tickets"

    def __str__(self):
        return str(self.summary)

    @property
    def budget_hours(self):
        budget_hours = 0

        for budget in self.budgetdata_set.all():
            budget_hours += budget.hours

        return budget_hours

    def is_project(self):
        """
        Return true if this is a project.

        This should match the projects_only manager logic.
        """
        return (self.itil_request_type == ItilRequestType.PROJECTS.value and
                self.project is None)


class TicketTracker(Ticket):
    tracker = FieldTracker()

    class Meta:
        proxy = True
        db_table = 'halo_ticket'



# Generated by Django 4.2.11 on 2024-10-18 11:27

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('halo', '0013_remove_ticket_start_time_remove_ticket_target_time_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='action',
            name='project',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='project_actions', to='halo.ticket'),
        ),
        migrations.AlterField(
            model_name='action',
            name='ticket',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='ticket_actions', to='halo.ticket'),
        ),
    ]



# Generated by Django 4.2.16 on 2025-03-20 12:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('halo', '0026_merge_20250314_1651'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ticket',
            name='appointment_type',
            field=models.IntegerField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='ticket',
            name='last_action_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]



# Generated by Django 4.2.11 on 2024-08-13 16:29

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='SyncJob',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('start_time', models.DateTimeField()),
                ('end_time', models.DateTimeField(blank=True, null=True)),
                ('entity_name', models.CharField(max_length=100)),
                ('synchronizer_class', models.CharField(blank=True, max_length=100, null=True)),
                ('added', models.PositiveIntegerField(null=True)),
                ('updated', models.PositiveIntegerField(null=True)),
                ('deleted', models.PositiveIntegerField(null=True)),
                ('skipped', models.PositiveIntegerField(null=True)),
                ('success', models.BooleanField(null=True)),
                ('message', models.TextField(blank=True, null=True)),
                ('sync_type', models.CharField(default='full', max_length=32)),
            ],
        ),
    ]



from djpsa.halo import models
from djpsa.halo.records import api
from djpsa.halo import sync


class SLASynchronizer(sync.HaloSynchronizer):

    model_class = models.SLATracker
    client_class = api.StatusAPI

    def _assign_field_data(self, instance, json_data):

        instance.id = json_data.get('id')
        instance.name = json_data.get('name')
        instance.hours_are_techs_local_time = \
            json_data.get('hoursaretechslocaltime', False)
        instance.response_reset = json_data.get('responsereset', False)
        instance.response_reset_approval = \
            json_data.get('response_reset_approval', False)
        instance.track_sla_fix_by_time = \
            json_data.get('trackslafixbytime', False)
        instance.track_sla_response_time = \
            json_data.get('trackslaresponsetime', False)
        instance.workday_id = json_data.get('workday_id')
        instance.auto_release_limit = json_data.get('autoreleaselimit')
        instance.auto_release_option = \
            json_data.get('autoreleaseoption', False)
        instance.status_after_first_warning = \
            json_data.get('statusafterfirstwarning')
        instance.status_after_second_warning = \
            json_data.get('statusaftersecondwarning')
        instance.status_after_auto_release = \
            json_data.get('statusafterautorelease')



# Generated by Django 4.2.16 on 2025-02-28 15:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('halo', '0023_action_non_billable_time_action_outcome_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='ticket',
            name='date_closed',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]



# Generated by Django 4.2.11 on 2024-09-20 07:46

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('halo', '0009_alter_halouser_options_alter_site_options_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='AgentTracker',
            fields=[
            ],
            options={
                'db_table': 'halo_agent',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('halo.agent',),
        ),
        migrations.CreateModel(
            name='AppointmentTracker',
            fields=[
            ],
            options={
                'db_table': 'halo_appointment',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('halo.appointment',),
        ),
        migrations.CreateModel(
            name='ClientTracker',
            fields=[
            ],
            options={
                'db_table': 'halo_client',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('halo.client',),
        ),
        migrations.CreateModel(
            name='HaloUserTracker',
            fields=[
            ],
            options={
                'db_table': 'halo_halouser',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('halo.halouser',),
        ),
        migrations.CreateModel(
            name='PriorityTracker',
            fields=[
            ],
            options={
                'db_table': 'halo_priority',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('halo.priority',),
        ),
        migrations.CreateModel(
            name='ProjectTracker',
            fields=[
            ],
            options={
                'db_table': 'halo_project',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('halo.project',),
        ),
        migrations.CreateModel(
            name='SiteTracker',
            fields=[
            ],
            options={
                'db_table': 'halo_site',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('halo.site',),
        ),
        migrations.CreateModel(
            name='SLATracker',
            fields=[
            ],
            options={
                'db_table': 'halo_sla',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('halo.sla',),
        ),
        migrations.CreateModel(
            name='StatusTracker',
            fields=[
            ],
            options={
                'db_table': 'halo_status',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('halo.status',),
        ),
        migrations.CreateModel(
            name='TicketTracker',
            fields=[
            ],
            options={
                'db_table': 'halo_ticket',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('halo.ticket',),
        ),
        migrations.RenameField(
            model_name='ticket',
            old_name='parent',
            new_name='project',
        ),
    ]



# Generated by Django 4.2.11 on 2024-08-15 14:10

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('halo', '0002_rename_isdisabled_agent_is_disabled_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ticket',
            name='agent',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='halo.agent'),
        ),
        migrations.AlterField(
            model_name='ticket',
            name='client',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='halo.client'),
        ),
        migrations.AlterField(
            model_name='ticket',
            name='priority',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='halo.priority'),
        ),
        migrations.AlterField(
            model_name='ticket',
            name='summary',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]
